﻿<#	
	.NOTES
	===========================================================================
	 Created with: 	Powershell 5
	 Created on:   	01.05.2020 15:00
	 Created by:   	Jörn Walter
	 Organization: 	Der Windows Papst
	 Filename:     	PSRemoting.ps1
	===========================================================================
	.DESCRIPTION
        Dieses Skript prüft ob ein Zertifikat aus Basis der Vorlage WSAN vorhanden ist, wenn ja dann wird WinRM an dieses Zertifikat gebunden. Ist keins vorhanden dann wird ein gpupdate ausgeführt
        und noch einmal geprüft. Das Skript beendet sich automatisch nach 90 Sekunden, um das System nicht einzuschränken.
        Zur Vorbereitung muss eine Zertifikatsvorlage automatisch ausgerollt werden. Genannt habe ich diese WSMAN.

        Aufbau einer sicheren Verbindung:
        $S = New-PSSession -ComputerName SRVSubCA.dwp.local -UseSSL
        Invoke-Command -Session $s {Get-Service -Name WinRM}
        Invoke-Command -Session $s {WinRM e winrm/config/listener}
        Invoke-Command -Session $s {winrm get winrm/config/service}
        Invoke-Command -Session $s {winrm get winrm/config/client/auth}
        Invoke-Command -Session $s {winrm get winrm/config/service/auth}
        Invoke-Command -Session $s {Set-Item -Path WSMan:\localhost\Service\Auth\Certificate -Value $true}
        Disconnect-PSSession -Session $s
        Remove-PSSession -Session $s
        Enter-PSSession -CN SRVSubCA.dwp.local -UseSSL

        $so = New-PsSessionOption –SkipCACheck -SkipCNCheck
        Enter-PSSession -Computername SRVSubCA.dwp.local -Credential (Get-Credential) -UseSSL -SessionOption $so

        # Diverse Management Befehle
        winrm set winrm/config/client/DefaultPorts '@{HTTP="5985"}'
        winrm set winrm/config/client/auth '@{Basic="true"}'
        winrm set winrm/config/service/DefaultPorts '@{HTTP="5985"}'
        winrm set winrm/config/service/auth '@{Basic="true"}'
        winrm get winrm/config
        winrm get winrm/config/client/auth
        winrm get winrm/config/service/auth
        winrm get winrm/config/service
        winrm get winrm/config/winrs

        winrm e winrm/config/listener
        Get-ChildItem WSMan:\Localhost\listener | Where -Property Keys -eq 'Transport=HTTP' | Remove-Item -Recurse
        Invoke-Command -Computername SRVSubCA.dwp.local -UseSSL -scriptBlock { Get-Service }
		Invoke-Command -CN SRVKMS.dwp.local,SRVSubCA.dwp.local -UseSSL -ScriptBlock {Get-Service spooler}
        Get-ChildItem WSMan:\localhost\Client\DefaultPorts
        Get-WSManInstance winrm/config/Listener –Enumerate
        Set-Item -Path WSMan:\localhost\Service\Auth\Certificate -Value $ true
#>
# Event Source wird angelegt sofern diese nicht vorhanden ist.
function Test-EventLog {
    Param(
        [Parameter(Mandatory=$true)]
        [string] $LogName
    )

    [System.Diagnostics.EventLog]::SourceExists($LogName)
}

$Event = Test-EventLog "PSRemoting"

If ($Event -eq $false) {

New-EventLog –LogName Application –Source "PSRemoting"

Write-Eventlog -LogName Application -Source "PSRemoting" -EntryType Information -EventId 0 -Message "Die Event-Source PSRemoting wurde angelegt."
}
# Zertifikatsspeicher festlegen
$certLocation = "LocalMachine\My"
# Funktion zum Auslesen der Erweiterung
function get_InstalledCertificateInfo($certLocation) {

	$array = gci Cert:\$certLocation | `

		select Thumbprint , `

		@{n="CN";e={($_.Subject).split("=")[1]}} , `
# Bei einem englischen OS -match Template
		@{n="IssuedfromTemplate";e={[regex]::match( ($_.extensions.Format(0) | ? { $_ -match "Vorlage" } ).split(",")[0] , '^(Vorlage\=)(\w.*)(\([0-9.]*(\)))$' ).Groups[2].value}}
	return $array
}
# Varibalen setzen
$WinRMcertFound = $false
$WinRMCertTemplateName = "WSMAN"

$timeout = New-TimeSpan -Seconds 90
$endTime = (Get-Date).Add($timeout)
# Aufgabe zur Ermittlung des Fingerprints der oben genannten Zertifikatsvorlage
# Ist ein Zertifikat basierend auf der Vorlage vorhanden wird es an den HTTPS Listener gebunden
do {

	if (Get_InstalledCertificateInfo $certLocation | ? {$_.IssuedfromTemplate -eq $WinRMCertTemplateName}){
		$WinRMCorporateCertThumbprint = ((Get_InstalledCertificateInfo $certLocation | ? {$_.IssuedfromTemplate -eq $WinRMCertTemplateName}).Thumbprint).replace(" ","").toupper()
		$WinRMCorporateCertCN = (Get_InstalledCertificateInfo $certLocation | ? {$_.IssuedfromTemplate -eq $WinRMCertTemplateName}).CN
        Write-Eventlog -LogName Application -Source "PSRemoting" -EntryType Information -EventId 1 -Message "Ein Zertifikat basierend auf das Template $WinRMCertTemplateName mit dem Thumbprint $WinRMCorporateCertThumbprint wurde im Computerspeicher gefunden."
		write-host "Ein Zertifikat basierend auf das Template $WinRMCertTemplateName mit dem Thumbprint $WinRMCorporateCertThumbprint wurde im Computerspeicher gefunden" -ForegroundColor Green
		$WinRMcertFound = $true

        Write-Host "Der HTTPS Listener wird neu konfiguriert" -ForegroundColor Cyan
        winrm delete winrm/config/Listener?Address=*+Transport=HTTPS
        New-Item WSMan:\localhost\Listener -Transport HTTPS -Address * -CertificateThumbprint $WinRMCorporateCertThumbprint -Force
        Write-Eventlog -LogName Application -Source "PSRemoting" -EntryType Information -EventId 2 -Message "Der Thumbprint $WinRMCorporateCertThumbprint wurde an den HTTPS Listener gebunden."
		Set-Item -force WSMan:\localhost\Service\Auth\Certificate $true
        Write-Eventlog -LogName Application -Source "PSRemoting" -EntryType Information -EventId 3 -Message "Die Service/Authentifizierung für Zertifikate wurde aktiviert."
	}
# Ist das Zertifkat noch nicht im Computerspeicher vorhanden, weil die Gruppenrichtlinien noch nicht aktualisiert wurden, wird das jetzt nachgeholt.
	else {
		write-host -ForegroundColor Cyan "Es wurde noch kein WinRM Zertifikat gefunden."
		write-host -ForegroundColor Magenta "Starte ein GPUPDATE /FORCE"
		gpupdate /force
	}
	
} until ($WinRMcertFound -eq $true -or ((Get-Date) -gt $endTime))
Restart-Service -Name WinRM -Force
exit